@@ -11,6 +11,7 @@ from functools import partialmethod
 from typing import TYPE_CHECKING, Callable, Dict, Iterable, Optional, Tuple, Type, Union
 
 from funcy import cached_property, first
+from git import GitCommandNotFound
 from pathspec.patterns import GitWildMatchPattern
 
 from scmrepo.base import Base
@@ -33,6 +34,8 @@ BackendCls = Type[BaseGitBackend]
 
 _LOW_PRIO_BACKENDS = ("gitpython",)
 
+_SKIPPABLE = (NotImplementedError, GitCommandNotFound)
+
 
 class GitBackends(Mapping):
     DEFAULT: Dict[str, BackendCls] = {
@@ -100,7 +103,7 @@ class Git(Base):
         for backend in self.backends.values():
             try:
                 return backend.dir
-            except NotImplementedError:
+            except _SKIPPABLE:
                 pass
         raise NotImplementedError
 
@@ -141,7 +144,7 @@ class Git(Base):
                 if rev:
                     repo.checkout(rev)
                 return repo
-            except NotImplementedError:
+            except _SKIPPABLE:
                 pass
         raise NoGitBackendError("clone")
 
@@ -284,7 +287,7 @@ class Git(Base):
                 self._last_backend = key
                 self.backends.move_to_end(key, last=False)
                 return result
-            except NotImplementedError:
+            except _SKIPPABLE:
                 pass
         raise NoGitBackendError(name)
 
@@ -302,7 +305,7 @@ class Git(Base):
                 backend.init(path, bare=bare)
                 # TODO: reuse created object instead of initializing a new one.
                 return cls(path)
-            except NotImplementedError:
+            except _SKIPPABLE:
                 pass
         raise NoGitBackendError("init")
 
@@ -327,19 +330,6 @@ class Git(Base):
         progress: Optional[Callable[["GitProgressEvent"], None]] = None,
         **kwargs,
     ) -> typing.Mapping[str, SyncStatus]:
-        from urllib.parse import urlparse
-
-        from .credentials import get_matching_helper_commands
-
-        if "dulwich" in kwargs.get("backends", self.backends.backends):
-            credentials_helper = any(
-                get_matching_helper_commands(url, self.dulwich.repo.get_config_stack())
-            )
-            parsed = urlparse(url)
-            ssh = parsed.scheme in ("git", "git+ssh", "ssh") or url.startswith("git@")
-            if credentials_helper or ssh:
-                kwargs["backends"] = ["dulwich"]
-
         return self._fetch_refspecs(
             url,
             refspecs,
