@@ -64,7 +64,12 @@ class BaseGitBackend(ABC):
         pass
 
     @abstractmethod
-    def add(self, paths: Union[str, Iterable[str]], update=False):
+    def add(
+        self,
+        paths: Union[str, Iterable[str]],
+        update: bool = False,
+        force: bool = False,
+    ):
         pass
 
     @abstractmethod
@@ -241,8 +246,7 @@ class BaseGitBackend(ABC):
         """Fetch refspecs from a remote Git repo.
 
         Args:
-            url: Remote repo Git URL (Note this must be a Git URL and not
-                a remote name).
+            url: Git remote name or absolute Git URL.
             refspecs: Iterable containing refspecs to fetch.
                 Note that this will not match subkeys.
             force: If True, local refs will be overwritten.
@@ -262,7 +266,7 @@ class BaseGitBackend(ABC):
         self,
         ref: str,
         message: Optional[str] = None,
-        include_untracked: Optional[bool] = False,
+        include_untracked: bool = False,
     ) -> Tuple[Optional[str], bool]:
         """Push a commit onto the specified stash.
 
