@@ -3,6 +3,7 @@ import locale
 import logging
 import os
 import stat
+from contextlib import closing
 from functools import partial
 from io import BytesIO, StringIO
 from typing import (
@@ -11,6 +12,7 @@ from typing import (
     Callable,
     Dict,
     Iterable,
+    Iterator,
     List,
     Mapping,
     Optional,
@@ -97,14 +99,32 @@ class DulwichProgressReporter(GitProgressReporter):
 
 
 def _get_ssh_vendor() -> "SSHVendor":
+    import sys
+
     from dulwich.client import SubprocessSSHVendor
 
-    from .asyncssh_vendor import AsyncSSHVendor
+    from .asyncssh_vendor import AsyncSSHVendor, get_unsupported_opts
 
     ssh_command = os.environ.get("GIT_SSH_COMMAND", os.environ.get("GIT_SSH"))
     if ssh_command:
         logger.debug("dulwich: Using environment GIT_SSH_COMMAND '%s'", ssh_command)
         return SubprocessSSHVendor()
+
+    if sys.platform == "win32" and os.environ.get("MSYSTEM"):
+        # see https://github.com/iterative/dvc/issues/7702
+        logger.debug(
+            "dulwich: native win32 Python inside MSYS2/git-bash, using MSYS2 OpenSSH"
+        )
+        return SubprocessSSHVendor()
+
+    default_config = os.path.expanduser(os.path.join("~", ".ssh", "config"))
+    unsupported = list(get_unsupported_opts([default_config]))
+    if unsupported:
+        logger.debug(
+            "dulwich: unsupported SSH config option(s) '%s', using system OpenSSH",
+            ", ".join(unsupported),
+        )
+        return SubprocessSSHVendor()
     return AsyncSSHVendor()
 
 
@@ -198,7 +218,9 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
                 repo = clone_from(depth=depth, branch=os.fsencode(shallow_branch))
             else:
                 repo = clone_from()
-            cls._set_default_tracking_branch(repo)
+
+            with closing(repo):
+                cls._set_default_tracking_branch(repo)
         except Exception as exc:
             raise CloneError(url, to_path) from exc
 
@@ -227,17 +249,37 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     def dir(self) -> str:
         return self.repo.commondir()
 
-    def add(self, paths: Union[str, Iterable[str]], update=False):
+    def add(
+        self,
+        paths: Union[str, Iterable[str]],
+        update: bool = False,
+        force: bool = False,
+    ):
         assert paths or update
 
-        if isinstance(paths, str):
-            paths = [paths]
+        paths = [paths] if isinstance(paths, str) else list(paths)
 
         if update and not paths:
             self.repo.stage(list(self.repo.open_index()))
             return
 
-        files: List[bytes] = []
+        files: List[bytes] = [
+            os.fsencode(fpath) for fpath in self._expand_paths(paths, force=force)
+        ]
+        if update:
+            index = self.repo.open_index()
+            if os.name == "nt":
+                # NOTE: we need git/unix separator to compare against index
+                # paths but repo.stage() expects to be called with OS paths
+                self.repo.stage(
+                    [fname for fname in files if fname.replace(b"\\", b"/") in index]
+                )
+            else:
+                self.repo.stage([fname for fname in files if fname in index])
+        else:
+            self.repo.stage(files)
+
+    def _expand_paths(self, paths: List[str], force: bool = False) -> Iterator[str]:
         for path in paths:
             if not os.path.isabs(path) and self._submodules:
                 # NOTE: If path is inside a submodule, Dulwich expects the
@@ -254,27 +296,15 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
                         )
                         break
             if os.path.isdir(path):
-                files.extend(
-                    os.fsencode(relpath(os.path.join(root, fpath), self.root_dir))
-                    for root, _, fs in os.walk(path)
-                    for fpath in fs
-                )
+                for root, _, fs in os.walk(path):
+                    for fpath in fs:
+                        rel = relpath(os.path.join(root, fpath), self.root_dir)
+                        if force or not self.ignore_manager.is_ignored(rel):
+                            yield rel
             else:
-                files.append(os.fsencode(relpath(path, self.root_dir)))
-
-        # NOTE: this doesn't check gitignore, same as GitPythonBackend.add
-        if update:
-            index = self.repo.open_index()
-            if os.name == "nt":
-                # NOTE: we need git/unix separator to compare against index
-                # paths but repo.stage() expects to be called with OS paths
-                self.repo.stage(
-                    [fname for fname in files if fname.replace(b"\\", b"/") in index]
-                )
-            else:
-                self.repo.stage([fname for fname in files if fname in index])
-        else:
-            self.repo.stage(files)
+                rel = relpath(path, self.root_dir)
+                if force or not self.ignore_manager.is_ignored(rel):
+                    yield rel
 
     def commit(self, msg: str, no_verify: bool = False):
         from dulwich.errors import CommitError
@@ -487,7 +517,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
     def get_refs_containing(self, rev: str, pattern: Optional[str] = None):
         raise NotImplementedError
 
-    def push_refspecs(
+    def push_refspecs(  # noqa: C901
         self,
         url: str,
         refspecs: Union[str, Iterable[str]],
@@ -547,10 +577,10 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             return new_refs
 
         try:
-            client.send_pack(
+            result = client.send_pack(
                 path,
                 update_refs,
-                self.repo.object_store.generate_pack_data,
+                generate_pack_data=self.repo.generate_pack_data,
                 progress=(DulwichProgressReporter(progress) if progress else None),
             )
         except (NotGitRepository, SendPackError) as exc:
@@ -558,6 +588,17 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
             raise SCMError(f"Git failed to push '{src}' to '{url}'") from exc
         except HTTPUnauthorized as exc:
             raise AuthError(url) from exc
+        if result.ref_status and any(
+            (value is not None) for value in result.ref_status.values()
+        ):
+            reasons = ", ".join(
+                (
+                    f"{os.fsdecode(ref)}: {reason}"
+                    for ref, reason in result.ref_status.items()
+                    if reason is not None
+                )
+            )
+            raise SCMError(f"Git failed to push some refs to '{url}' ({reasons})")
         return change_result
 
     def fetch_refspecs(
@@ -573,6 +614,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         from dulwich.errors import NotGitRepository
         from dulwich.objectspec import parse_reftuples
         from dulwich.porcelain import DivergedBranches, check_diverged, get_remote_repo
+        from dulwich.refs import DictRefsContainer
 
         fetch_refs = []
 
@@ -582,7 +624,7 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         ) -> List[bytes]:
             fetch_refs.extend(
                 parse_reftuples(
-                    remote_refs,
+                    DictRefsContainer(remote_refs),
                     self.repo.refs,
                     os.fsencode(refspecs)
                     if isinstance(refspecs, str)
@@ -651,12 +693,18 @@ class DulwichBackend(BaseGitBackend):  # pylint:disable=abstract-method
         self,
         ref: str,
         message: Optional[str] = None,
-        include_untracked: Optional[bool] = False,
+        include_untracked: bool = False,
     ) -> Tuple[Optional[str], bool]:
         from dulwich.repo import InvalidUserIdentity
 
         from scmrepo.git import Stash
 
+        # dulwich will silently generate an empty stash commit if there is
+        # nothing to stash, we check status here to get consistent behavior
+        # across backends
+        if not self.is_dirty(untracked_files=include_untracked):
+            return None, False
+
         if include_untracked or ref == Stash.DEFAULT_STASH:
             # dulwich stash.push does not support include_untracked and does
             # not touch working tree
